import React from 'react';
import { INode, IEdge, ContentHandler } from './document';
import { IState } from './chartEngine';
interface IPoint {
    x: number;
    y: number;
    id: string;
}
interface IMappedEdge {
    source: IPoint;
    target: IPoint;
}
interface IChartState {
    x: number;
    y: number;
    opacity: number;
    nodeDatum: any;
}
interface IChartProp {
    contentHandler: ContentHandler;
    nodes: INode[];
    edges: IEdge[];
    setChartState: Function;
    width: number;
    height: number;
    layout: Function;
}
export declare class Chart extends React.Component<IChartProp, IChartState> {
    myRef: React.RefObject<HTMLDivElement>;
    datum: any;
    mouse: {
        x: number;
        y: number;
    };
    mousePage: {
        x: number;
        y: number;
    };
    starting: {
        groupName: string;
        from: string;
        point: IPoint;
    };
    tooltip: any;
    textHeight: number;
    circleHeight: number;
    circleRadius: number;
    bars: any;
    svg: any;
    transform: any;
    link: any;
    mouseLink: any;
    g: any;
    inputPorts: Set<string>;
    outputPorts: Set<string>;
    inputRequriements: {
        [key: string]: any;
    };
    outputMeta: {
        [key: string]: any;
    };
    portTypes: {
        [key: string]: string[][];
    };
    isDirty: boolean;
    constructor(props: IChartProp);
    portMap(): void;
    translateCorr(portStr: string, from?: boolean): IPoint;
    edgeMap(d: IEdge): IMappedEdge;
    edgeData(): IMappedEdge[];
    nodeReLayout(sender: ContentHandler, inputs: void): void;
    addNewNode(sender: ContentHandler, inputs: INode): void;
    componentWillUnmount(): void;
    componentDidMount(): void;
    drawCircles(): void;
    drawLinks(): void;
    componentDidUpdate(): void;
    reLayout(): void;
    /**
     * Update the charts, also send back to server to reculcate the columns,
     * types and UI schema etc
     * @param nodes
     * @param edges
     */
    fullUpdate(nodes: INode[], edges: IEdge[]): void;
    nodeEditorUpdate(state: IState): void;
    renderEditor(): void;
    _updateInputs(json: string, nodes: INode[], edges: IEdge[]): void;
    connectionUpdate(): void;
    render(): JSX.Element;
}
export {};
