import React from 'react';
import { IEdge, INode, ContentHandler, IChartInput } from './document';
interface IProps {
    contentHandler: ContentHandler;
}
export interface IState {
    height: number;
    width: number;
    nodes: INode[];
    edges: IEdge[];
}
/**
 * Serialize the graph to list of INode objects that can be saved into files.
 * UI related infomration is stripped out. The special OUTPUT_COLLECTOR node is removed
 * @param nodes
 * @param edges
 * @returns list of INode that can be serialized
 */
export declare function exportWorkFlowNodes(nodes: INode[], edges: IEdge[]): INode[];
export declare class ChartEngine extends React.Component<IProps, IState> {
    constructor(props: IProps);
    componentWillUnmount(): void;
    /**
     *  handle the raw graph nodes and edge changes
     *  recalculate the layout
     * @param sender
     * @param inputs
     */
    contentChangeHandler(sender: ContentHandler, inputs: IChartInput): void;
    saveCacheHandler(sender: ContentHandler): void;
    /**
     *  handle importing raw graph nodes and edges,
     * resolve the name collision,
     *  recalculate the layout, no size information
     * @param sender
     * @param inputs
     */
    contentIncludeHandler(sender: ContentHandler, inputs: IChartInput): void;
    /**
     *  handle the raw graph nodes and edge changes
     *  recalculate the layout, no size information
     * @param sender
     * @param inputs
     */
    contentResetHandler(sender: ContentHandler, inputs: IChartInput): void;
    /**
     *
     *  handle the raw graph nodes and edge changes,
     *  just update the nodes and edges, no layout updates
     * @param sender
     * @param inputs
     */
    stateUpdateHandler(sender: ContentHandler, inputs: IChartInput): void;
    /**
     *
     *  handle the raw graph nodes and edge changes,
     *  just update the nodes and edges, no layout updates
     * @param sender
     * @param inputs
     */
    stateHandleResize(sender: ContentHandler, inputs: IChartInput): void;
    /**
     * Compute the coordinates of the nodes so it fits the give width and height
     * @param nodes
     * @param edges
     * @param transform  transform object returned from d3.zoom
     * @param width if specified, it overrides the state.width
     * @param height
     */
    private _updateLayout;
    /**
     * relayout the nodes and edges
     *
     * @param nodes
     * @param edges
     * @param transform
     */
    layout(nodes: INode[], edges: IEdge[], transform: any): void;
    private _fixDynamicPorts;
    private _fixOutputCollectorPorts;
    updateWorkFlow(state: IState, update?: boolean): void;
    render(): JSX.Element;
}
export {};
