import { DocumentWidget, DocumentRegistry, ABCWidgetFactory } from '@jupyterlab/docregistry';
import { MainView } from './mainComponent';
import { IEditorProp } from './nodeEditor';
import { Signal } from '@lumino/signaling';
import { MainAreaWidget } from '@jupyterlab/apputils';
import { WidgetModel } from '@jupyter-widgets/base';
import { CommandRegistry } from '@lumino/commands';
export declare class ContentHandler {
    private _outputs;
    context: DocumentRegistry.Context;
    private _contentChanged;
    private _contentReset;
    private _includeContent;
    private _chartStateUpdate;
    private _sizeStateUpdate;
    private _runGraph;
    private _cleanResult;
    private _updateEditor;
    private _saveCache;
    private _nodeAdded;
    private _privateCopy;
    private _reLayout;
    private _aspectRatio;
    private _commandRegistry;
    set commandRegistry(commandRegistry: CommandRegistry);
    get commandRegistry(): CommandRegistry;
    get updateEditor(): Signal<ContentHandler, IEditorProp>;
    get runGraph(): Signal<ContentHandler, void>;
    get saveCache(): Signal<ContentHandler, void>;
    get cleanResult(): Signal<ContentHandler, void>;
    get reLayoutSignal(): Signal<ContentHandler, void>;
    set aspectRatio(aspectRatio: number);
    get aspectRatio(): number;
    set outputs(outputs: string[]);
    get outputs(): string[];
    get reLayoutSignalInstance(): Signal<ContentHandler, void>;
    get privateCopy(): WidgetModel;
    get nodeAddedSignal(): Signal<ContentHandler, INode>;
    get contentChanged(): Signal<ContentHandler, IChartInput>;
    get contentReset(): Signal<ContentHandler, IChartInput>;
    get includeContent(): Signal<ContentHandler, IChartInput>;
    get chartStateUpdate(): Signal<ContentHandler, IChartInput>;
    get sizeStateUpdate(): Signal<ContentHandler, IChartInput>;
    setPrivateCopy(widgetModel: WidgetModel): void;
    constructor(context: DocumentRegistry.Context);
    /**
     * Write graph info into the model
     * @param content
     */
    update(content: string): void;
    /**
     *
     * Load the file from the model
     * And covert it to include UI elements
     * Send to render
     * @param width, the width of the svg area
     * @param height, the width of the svg area
     */
    emit(width: number, height: number): void;
    /**
     * Use the raw object from the file as input,  add UI information (UI schema, required, output_meta etc) to it
     *
     * @param objContent
     * @param width
     * @param height
     */
    renderGraph(objContent: any, width?: number, height?: number): Promise<void>;
    private _onContentChanged;
}
export interface IPort {
    name: string;
    type: string[][];
    dynamic?: boolean;
}
export interface INode {
    parentIds?: string[];
    width?: number;
    x?: number;
    y?: number;
    id: string;
    type: string;
    schema: any;
    ui: any;
    conf: any;
    inputs: IPort[];
    outputs: IPort[];
    filepath?: string;
    module?: string;
    required: any;
    output_meta: any;
    busy?: boolean;
}
export interface IEdge {
    from: string;
    to: string;
}
export interface IChartInput {
    nodes: INode[];
    edges: IEdge[];
    width?: number;
    height?: number;
}
export interface IAllNodes {
    [key: string]: INode[];
}
export declare class GquantWidget extends DocumentWidget<MainAreaWidget<MainView>> {
    constructor(options: DocumentWidget.IOptions<MainAreaWidget<MainView>>);
    get contentHandler(): ContentHandler;
    readonly context: DocumentRegistry.Context;
}
/**
 * A widget factory for drawio.
 */
export declare class GquantFactory extends ABCWidgetFactory<GquantWidget, DocumentRegistry.IModel> {
    /**
     * Create a new widget given a context.
     */
    constructor(options: DocumentRegistry.IWidgetFactoryOptions);
    protected createNewWidget(context: DocumentRegistry.Context): GquantWidget;
}
