import { DocumentWidget, ABCWidgetFactory } from '@jupyterlab/docregistry';
import { MainView } from './mainComponent';
import { requestAPI } from './gquantlab';
import YAML from 'yaml';
import { Signal } from '@lumino/signaling';
import { MainAreaWidget } from '@jupyterlab/apputils';
export class ContentHandler {
    constructor(context) {
        // this one will relayout the graph
        this._contentChanged = new Signal(this);
        // this reset the content, leave the size as before
        this._contentReset = new Signal(this);
        // this add extra graph to current graph, leave the size as before
        this._includeContent = new Signal(this);
        // this one just update the graphs, no relayout
        this._chartStateUpdate = new Signal(this);
        // this one just update the size of the chart
        this._sizeStateUpdate = new Signal(this);
        this._runGraph = new Signal(this);
        this._cleanResult = new Signal(this);
        this._updateEditor = new Signal(this);
        // signal used to sync the graph status to server
        this._saveCache = new Signal(this);
        // create a signal that emits the added node command
        this._nodeAdded = new Signal(this);
        // create a signal that emits the relayout command
        this._reLayout = new Signal(this);
        this._aspectRatio = 0.3;
        this.context = context;
        //this.context.model.contentChanged.connect(this._onContentChanged, this);
        //this.emit();
    }
    set commandRegistry(commandRegistry) {
        this._commandRegistry = commandRegistry;
    }
    get commandRegistry() {
        return this._commandRegistry;
    }
    get updateEditor() {
        return this._updateEditor;
    }
    get runGraph() {
        return this._runGraph;
    }
    get saveCache() {
        return this._saveCache;
    }
    get cleanResult() {
        return this._cleanResult;
    }
    get reLayoutSignal() {
        return this._reLayout;
    }
    set aspectRatio(aspectRatio) {
        this._aspectRatio = aspectRatio;
    }
    get aspectRatio() {
        return this._aspectRatio;
    }
    set outputs(outputs) {
        this._outputs = outputs;
    }
    get outputs() {
        return this._outputs ? this._outputs : [];
    }
    get reLayoutSignalInstance() {
        return this._reLayout;
    }
    get privateCopy() {
        return this._privateCopy;
    }
    get nodeAddedSignal() {
        return this._nodeAdded;
    }
    get contentChanged() {
        return this._contentChanged;
    }
    get contentReset() {
        return this._contentReset;
    }
    get includeContent() {
        return this._includeContent;
    }
    get chartStateUpdate() {
        return this._chartStateUpdate;
    }
    get sizeStateUpdate() {
        return this._sizeStateUpdate;
    }
    setPrivateCopy(widgetModel) {
        if (!widgetModel) {
            return;
        }
        this._privateCopy = widgetModel;
    }
    /**
     * Write graph info into the model
     * @param content
     */
    update(content) {
        if (this.context) {
            this.context.model.fromString(content);
        }
    }
    /**
     *
     * Load the file from the model
     * And covert it to include UI elements
     * Send to render
     * @param width, the width of the svg area
     * @param height, the width of the svg area
     */
    emit(width, height) {
        this._onContentChanged(width, height);
    }
    /**
     * Use the raw object from the file as input,  add UI information (UI schema, required, output_meta etc) to it
     *
     * @param objContent
     * @param width
     * @param height
     */
    async renderGraph(objContent, width, height) {
        const jsonString = JSON.stringify(objContent);
        // this.context.model.contentChanged.connect(this._onContentChanged, this);
        const workflows = await requestAPI('load_graph', {
            body: jsonString,
            method: 'POST'
        });
        if (width) {
            workflows.width = width;
        }
        if (height) {
            workflows.height = height;
        }
        this._contentChanged.emit(workflows);
    }
    _onContentChanged(width, height) {
        console.log('content chagned');
        const refreshContent = async () => {
            if (this.context === null) {
                return;
            }
            await this.context.ready;
            const yamlContent = this.context.model.toString();
            console.log('model path', this.context.path);
            const objContent = YAML.parse(yamlContent);
            this.renderGraph(objContent, width, height);
        };
        refreshContent();
    }
}
export class GquantWidget extends DocumentWidget {
    constructor(options) {
        super(Object.assign({}, options));
        this.context = options['context'];
    }
    get contentHandler() {
        const mainAreaWidget = this.content;
        const mainView = mainAreaWidget.content;
        return mainView.contentHandler;
    }
}
/**
 * A widget factory for drawio.
 */
export class GquantFactory extends ABCWidgetFactory {
    /**
     * Create a new widget given a context.
     */
    constructor(options) {
        super(options);
    }
    createNewWidget(context) {
        const contentHandler = new ContentHandler(context);
        const mainView = new MainView(contentHandler);
        const widget = new MainAreaWidget({ content: mainView });
        widget.toolbar.hide();
        return new GquantWidget({ context, content: widget });
    }
}
