/* eslint-disable @typescript-eslint/camelcase */
import { DOMWidgetModel, DOMWidgetView, ViewList } from '@jupyter-widgets/base';
import * as widgets from '@jupyter-widgets/base';
import { MODULE_NAME, MODULE_VERSION } from './version';
import { requestAPI } from './gquantlab';
import { ContentHandler } from './document';
import { MainView } from './mainComponent';
import { Panel } from '@lumino/widgets';
import { CommandRegistry } from '@lumino/commands';
// import { Message } from '@lumino/messaging';
import { Toolbar, CommandToolbarButton } from '@jupyterlab/apputils';
import { COMMAND_TOOL_BAR_RELAYOUT, COMMAND_TOOL_BAR_EXECUTE, COMMAND_TOOL_BAR_CLEAN, COMMAND_TOOL_BAR_OPEN_NEW_FILE, COMMAND_TOOL_BAR_CONVERT_CELL_TO_FILE, COMMAND_TOOL_BAR_INCLUDE_NEW_FILE, COMMAND_TOOL_BAR_SHOW_LOG, setupToolBarCommands } from './commands';
import { ContextMenuSvg } from '@jupyterlab/ui-components';
import { setupContextMenu } from '.';
export class GQuantModel extends DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: GQuantModel.model_name, _model_module: GQuantModel.model_module, _model_module_version: GQuantModel.model_module_version, _view_name: GQuantModel.view_name, _view_module: GQuantModel.view_module, _view_module_version: GQuantModel.view_module_version, value: [], cache: { nodes: [], edges: [] }, sub: null });
    }
}
GQuantModel.serializers = Object.assign(Object.assign({}, DOMWidgetModel.serializers), { sub: { deserialize: widgets.unpack_models } });
GQuantModel.model_name = 'GQuantModel';
GQuantModel.model_module = MODULE_NAME;
GQuantModel.model_module_version = MODULE_VERSION;
GQuantModel.view_name = 'GQuantView'; // Set to null if no view
GQuantModel.view_module = MODULE_NAME; // Set to null if no view
GQuantModel.view_module_version = MODULE_VERSION;
export class GQuantView extends DOMWidgetView {
    addCommands(commands, toolBar) {
        const item0 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_RELAYOUT
        });
        const item1 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_EXECUTE
        });
        const item2 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_CLEAN
        });
        const item3 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_OPEN_NEW_FILE
        });
        const item4 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_CONVERT_CELL_TO_FILE
        });
        const item5 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_INCLUDE_NEW_FILE
        });
        const item6 = new CommandToolbarButton({
            commands: commands,
            id: COMMAND_TOOL_BAR_SHOW_LOG
        });
        toolBar.addItem('relayout', item0);
        toolBar.addItem('run', item1);
        toolBar.addItem('clean', item2);
        toolBar.addItem('open', item3);
        toolBar.addItem('add', item5);
        toolBar.addItem('create', item4);
        toolBar.addItem('log', item6);
    }
    render() {
        this._contentHandler = new ContentHandler(null);
        const commands = new CommandRegistry();
        this._contentHandler.commandRegistry = commands; // use its local command registry
        this._contentHandler.runGraph.connect(this.run, this);
        this._contentHandler.cleanResult.connect(this.clean, this);
        const pane = new Panel();
        this._widget = new MainView(this._contentHandler);
        const toolBar = new Toolbar();
        pane.addWidget(this._widget);
        pane.addWidget(toolBar);
        this.pWidget = pane;
        this._contentHandler.renderGraph(this.model.get('value'));
        this._contentHandler.setPrivateCopy(this.model);
        this.model.on('change:cache', this.cache_changed, this);
        this.views = new ViewList(this.addView, null, this);
        this.model.on('change:sub', this.sub_changed, this);
        setupToolBarCommands(commands, this._contentHandler, GQuantView.browserFactory, GQuantView.apps.commands, GQuantView.apps);
        this.addCommands(commands, toolBar);
        this.createContexMenu();
    }
    run() {
        this.send({
            event: 'run'
        });
    }
    clean() {
        const cache = this.model.get('cache');
        cache.nodes.forEach((node) => {
            node.busy = false;
        });
        this._contentHandler.chartStateUpdate.emit(cache);
        this._contentHandler.privateCopy.set('cache', cache);
        this.send({
            event: 'clean'
        });
    }
    sub_changed(model, value) {
        const subView = this.create_child_view(value);
        subView.then(view => {
            const pane = this.pWidget;
            if (pane.widgets.length === 3) {
                pane.layout.removeWidget(pane.widgets[2]);
            }
            pane.insertWidget(2, view.pWidget);
            // const message: Message = new Message('resize');
            //this._widget.processMessage(message);
        });
        this.views.update([value]).then(() => {
            const pane = this.pWidget;
            if (pane.widgets.length === 3) {
                const w = pane.widgets[2];
                const cell = this._widget.parent.parent.parent.parent.parent
                    .parent;
                const _width = cell.inputArea.editorWidget.node.clientWidth - 5;
                w.node.style.width = _width + 'px';
            }
        });
    }
    getFigureSize() {
        const figureSize = this.el.getBoundingClientRect();
        return figureSize;
    }
    addView(model) {
        return this.create_child_view(model);
    }
    processPhosphorMessage(msg) {
        super.processPhosphorMessage.apply(this, msg);
        switch (msg.type) {
            case 'resize':
            case 'after-show':
            case 'after-attach':
                if (this.pWidget.isVisible) {
                    console.log(this.getFigureSize());
                }
                break;
        }
    }
    cache_changed() {
        const cache = this.model.get('cache');
        if ('register' in cache) {
            const payload = cache['register'];
            const result = requestAPI('register_node', {
                body: JSON.stringify(payload),
                method: 'POST'
            });
            result.then(data => {
                console.log(data);
            });
            delete cache['register'];
        }
        this._contentHandler.chartStateUpdate.emit(cache);
    }
    createContexMenu() {
        const commands = GQuantView.apps.commands;
        const contextMenu = new ContextMenuSvg({ commands });
        setupContextMenu(contextMenu, commands, null);
        const createOuputView = 'notebook:create-output-view';
        contextMenu.addItem({
            type: 'separator',
            selector: '.jp-GQuant'
        });
        contextMenu.addItem({
            command: createOuputView,
            selector: '.jp-GQuant'
        });
        this.pWidget.node.addEventListener('contextmenu', (event) => {
            if (contextMenu.open(event)) {
                event.preventDefault();
                event.stopPropagation();
            }
        });
    }
}
GQuantView.apps = null;
GQuantView.browserFactory = null;
